#include <Servo.h>

// Button and feeding servo configuration
const int buttonPin = 3;
const int feedingServoPin = 11;

const unsigned long debounceDelay = 40;
int startPosition = 0;
int targetPosition = 75;

// Joystick and tilt/pan servos configuration

// Constants for tilt
const int tiltServoPin = 10;
const int tiltJoystickPin = A3;

const int minTiltAngle = 0;
const int maxTiltAngle = 44;
const int startTiltAngle = 5;

// Constants for pan
const int panServoPin = 13;
const int panJoystickPin = A2;

const int minPanAngle = 0;
const int maxPanAngle = 180;
const int startPanAngle = 90;

// ESC configuration
const int ESC_A_Pin = 12;
const int ESC_B_Pin = 9;

const int potAPin = A1;
const int potBPin = A0;

const int ESC_A_Min = 1080; // Min pulse width
const int ESC_A_Max = 1170; // Max pulse width

const int ESC_B_Min = 1130; // Min pulse width
const int ESC_B_Max = 1253; // Max pulse width
const int ESC_Arm_Delay = 2000;

// Servo speed
const unsigned long updateInterval = 20; // 20 milliseconds
const int angleChange = 2;

// Create servo objects
Servo feedingServo;
Servo tiltServo;
Servo panServo;
Servo ESC_A;
Servo ESC_B;

// Variables for button press handling
volatile bool buttonPressed = false;
unsigned long lastDebounceTime = 0;

// Variables to store the last servo angles
int lastTiltAngle = startTiltAngle;
int lastPanAngle = startPanAngle;

// Variable to store the last update time
unsigned long lastUpdateTime = 0;

void setup() {
  pinMode(buttonPin, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(buttonPin), onButtonPress, FALLING);
  feedingServo.attach(feedingServoPin);
  feedingServo.write(startPosition);

  tiltServo.attach(tiltServoPin);
  panServo.attach(panServoPin);

  tiltServo.write(startTiltAngle);
  panServo.write(startPanAngle);

  ESC_A.attach(ESC_A_Pin);
  ESC_B.attach(ESC_B_Pin);

  // Arm the ESCs
  ESC_A.writeMicroseconds(ESC_A_Min);
  ESC_B.writeMicroseconds(ESC_B_Min);
  delay(ESC_Arm_Delay);

  Serial.begin(9600);
}

void loop() {
  if (buttonPressed) {
    if ((millis() - lastDebounceTime) > debounceDelay) {
      buttonPressed = false;
      feedingServo.write(startPosition+5);
      delay(800);      
      feedingServo.write(targetPosition);
      delay(200);
      feedingServo.write(targetPosition-12);
      delay(200);
      feedingServo.write(targetPosition);
      delay(1200);
      feedingServo.write(startPosition);
    }
  }

  int tiltJoystickValue = analogRead(tiltJoystickPin);
  int panJoystickValue = analogRead(panJoystickPin);

  unsigned long currentTime = millis();

  if (currentTime - lastUpdateTime >= updateInterval) {
    if (tiltJoystickValue > 800) {
      lastTiltAngle = min(lastTiltAngle + angleChange, maxTiltAngle);
    } else if (tiltJoystickValue < 200) {
      lastTiltAngle = max(lastTiltAngle - angleChange, minTiltAngle);
    }
    tiltServo.write(lastTiltAngle);

        if (panJoystickValue > 800) {
      lastPanAngle = min(lastPanAngle + angleChange, maxPanAngle);
    } else if (panJoystickValue < 200) {
      lastPanAngle = max(lastPanAngle - angleChange, minPanAngle);
    }
    panServo.write(lastPanAngle);

    lastUpdateTime = currentTime;

    Serial.print("Tilt angle: ");
    Serial.print(lastTiltAngle);
    Serial.print(", Pan angle: ");
    Serial.println(lastPanAngle);
  }

  // Read potentiometer values
  int potAValue = analogRead(potAPin);
  int potBValue = analogRead(potBPin);

  // Map potentiometer values to ESC range
  int ESC_A_Value = map(potAValue, 0, 1023, ESC_A_Min, ESC_A_Max);
  int ESC_B_Value = map(potBValue, 0, 1023, ESC_B_Min, ESC_B_Max);

  // Write the mapped values to the ESCs
  ESC_A.writeMicroseconds(ESC_A_Value);
  ESC_B.writeMicroseconds(ESC_B_Value);

  // Print the mapped values for debugging purposes
  Serial.print("ESC A Value: ");
  Serial.print(ESC_A_Value);
  Serial.print(", ESC B Value: ");
  Serial.println(ESC_B_Value);

  delay(15);
}

void onButtonPress() {
  if ((millis() - lastDebounceTime) > debounceDelay) {
    lastDebounceTime = millis();
    buttonPressed = true;
  }
}

